/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation.valueprovider;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Base class for {@link IAnnotationValueProvider}s.
 * 
 * @author diewald, barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 92072FF28C0A6055490A33DB8F1A062B
 */
public abstract class ValueProviderBase<T extends IAnnotatedSpecification> implements
		IAnnotationValueProvider<T> {

	/**
	 * {@link EClass} of {@link IAnnotatedSpecification} that contains the respective structural
	 * feature and that is managed by this {@link IAnnotationValueProvider}.
	 */
	protected final EClass annotatedSpecificationEClass;

	/**
	 * Map: instanceKey -> Fixed input choice to be offered in a combo box. The semantics of the map
	 * value is as follows (see {@link IAnnotationValueProvider} for the definition of instance
	 * keys).
	 * <ul>
	 * <li> {@code fixedInputChoice[key] == null} -> do not use fixed values, provide text input
	 * cell.</li>
	 * <li> {@code !fixedInputChoice[key].isEmpty()} -> provide a fixed set of choices in an input
	 * box.
	 * <li> {@code fixedInputChoice[key].isEmpty()} -> dynamically manage the set of provided values,
	 * i.e., the combo box is editable and enables the user to add new values and remove existing
	 * choices (by removing all instances of them).</li>
	 * <li>For enum types, the set of inputs is automatically initialized from the underlying
	 * {@link EEnum} type definition.</li>
	 * </ul>
	 */
	private Map<String, List<String>> fixedInputChoiceMap;

	/**
	 * Map: instanceKey -> Current input choice, computed according to the initialization of
	 * {@link #fixedInputChoiceMap}.
	 */
	private Map<String, Collection<String>> currentInputChoiceMap;

	/**
	 * Constructs a new {@link IAnnotationValueProvider} for an annotation specified in a given
	 * {@link EClass}.
	 */
	public ValueProviderBase(EClass annotatedSpecificationEClass) {
		this.annotatedSpecificationEClass = annotatedSpecificationEClass;

		fixedInputChoiceMap = new HashMap<String, List<String>>();
		currentInputChoiceMap = new HashMap<String, Collection<String>>();
	}

	/** {@inheritDoc} */
	@Override
	public boolean setAnnotationName(String name, T specification) {
		// Renaming this annotation is not supported by default.
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isHidden(T specification) {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canEdit(T specification) {
		return canEdit(specification, DEFAULT_KEY);
	}

	/** {@inheritDoc} */
	@Override
	public <U> U getAnnotationValue(T specification, String instanceKey) throws Exception {
		return getAnnotationValue(specification);
	}

	/** {@inheritDoc} */
	@Override
	public void setAnnotationValue(String value, T specification) throws Exception {
		setAnnotationValue(value, specification, null);
	}

	/** {@inheritDoc} */
	@Override
	public <U> void setAnnotationValue(U value, T specification) throws Exception {
		setAnnotationValue(value, specification, null);
	}

	/** {@inheritDoc} */
	@Override
	public void setAnnotationValue(String value, T specification, String instanceKey)
			throws Exception {
		throw new Exception(
				"setAnnotationValue(String) is not supported / has not been implemented for annotations of type " +
						specification.getClass());
	}

	/** {@inheritDoc} */
	@Override
	public <U> void setAnnotationValue(U value, T specification, String instanceKey)
			throws Exception {
		throw new Exception(
				"setAnnotationValue(U) is not supported / has not been implemented for annotations of type " +
						specification.getClass());
	}

	/** {@inheritDoc} */
	@Override
	public List<Class<? extends EObject>> excludeModelElementTypeFromAnnotatedSpecification() {
		return null;
	}

	/**
	 * Add decorations to an annotation when its instance is newly created by
	 * {@link #getAnnotatedSpecificationForModelElement(IModelElement)}.
	 * 
	 * @param specification
	 *            annotation to be decorated
	 */
	protected void decorateAnnotationSpecification(T specification) throws Exception {
		decorateAnnotationSpecification(specification, null);
	}

	/**
	 * Add decorations to an annotation when its instance is newly created by
	 * {@link #getAnnotatedSpecificationForModelElement(IModelElement)}.
	 * 
	 * @param specification
	 *            annotation to be decorated
	 * @param instanceKey
	 *            instanceKey of annotation to be decorated
	 */
	protected void decorateAnnotationSpecification(T specification, String instanceKey)
			throws Exception {
		// No decoration
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public T getAnnotatedSpecificationForModelElement(IModelElement element) throws Exception {

		Class<IAnnotatedSpecification> instanceClass =
				(Class<IAnnotatedSpecification>)annotatedSpecificationEClass.getInstanceClass();
		T rval = (T)pickFirstInstanceOf(instanceClass, element.getSpecifications());

		if(rval != null) {
			return rval;
		}

		// Create annotation
		EFactory eFactory = annotatedSpecificationEClass.getEPackage().getEFactoryInstance();
		T specification = (T)eFactory.create(annotatedSpecificationEClass);

		if(getInstanceKeys(specification).isEmpty()) {
			decorateAnnotationSpecification(specification);
		} else {
			for(String key : getInstanceKeys(specification)) {
				try {
					decorateAnnotationSpecification(specification, key);
				} catch(Exception e) {
					// Some of the keys might not be backed by an implementation that needs to
					// "decorate" the underlying specification object (e.g., a custom key that
					// represents a calculated, volatile value. Hence, just ignore invalid keys
					// during this setup phase. The other methods (get/set/...) will throw an
					// Exception on invalid keys.
				}
			}
		}

		// Hook specification to model element
		element.addSpecification(specification);

		return specification;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<T> getAnnotationClazz() {
		return (Class<T>)annotatedSpecificationEClass.getInstanceClass();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<String> getInstanceKeys(T specification) {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public boolean allowsDynamicAnnotationInstances() {
		return false;
	}

	/** Returns the {@link #fixedInputChoiceMap}. */
	public Map<String, List<String>> getFixedInputChoiceMap() {
		return fixedInputChoiceMap;
	}

	/** Returns the {@link #currentInputChoiceMap}. */
	public Map<String, Collection<String>> getCurrentInputChoiceMap() {
		return currentInputChoiceMap;
	}

	/** Sets the {@link #currentInputChoiceMap}. */
	public void setCurrentInputChoiceMap(Map<String, Collection<String>> currentInputChoiceMap) {
		this.currentInputChoiceMap = currentInputChoiceMap;
	}

	/**
	 * Updates the current input choice, and returns the actual value. Called from
	 * {@link #setAnnotationValue(String, IAnnotatedSpecification)}. In particular, this
	 * method handles the removal of input choices from if these are dynamically managed (i.e.,
	 * based on the accumulated user input).
	 * 
	 * @param newValue
	 *            New value is that is about to be set in
	 *            {@link #setAnnotationValue(String, IAnnotatedSpecification)} and that is required
	 *            to update the input choice.
	 * @param specification
	 *            {@link IAnnotatedSpecification} for which the input choice is to be updated
	 * @param instanceKey
	 *            Key identifying annotation instance of {@code specification} to be used.
	 * 
	 * @return Currently selected choice.
	 */
	protected String determineCurrentAnnotationValueAndUpdateInputChoice(String newValue,
			T specification, String instanceKey) throws Exception {

		Collection<String> currentInputChoice = currentInputChoiceMap.get(instanceKey);
		boolean noValue = newValue == null || newValue.isEmpty();
		if(noValue && currentInputChoice != null && !currentInputChoice.isEmpty()) {

			// Only one entry left: do not remove
			if(currentInputChoice.size() == 1) {
				return currentInputChoice.iterator().next();
			}

			// 1st entry is to be removed: Choose 2nd entry as new value
			String currentValue = getAnnotationValue(specification);

			Iterator<String> iter = currentInputChoice.iterator();
			if(iter.next().equals(currentValue)) {
				// Skip one entry
				String rval = iter.next();
				currentInputChoice.remove(currentValue);
				return rval;
			}

			// Otherwise, return the entry just before the entry to be removed.
			String rval = null;

			for(String s : currentInputChoiceMap.get(instanceKey)) {
				if(s != null && s.equals(currentValue)) {
					currentInputChoice.remove(currentValue);
					return rval;
				}
				rval = s;
			}

			return rval;
		}
		return null;
	}
}
