/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.constraint;

import static org.fortiss.tooling.kernel.constraint.ConstraintMessage.createConsistencyIdViolation;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.base.ConstraintCheckerBase;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * <p>
 * {@link IConstraintChecker} to check whether in the tree below a given root {@link EObject}, the
 * IDs provided by {@link IIdLabeled} is identical to the XMI ID stored in the corresponding
 * {@link XMLResource}.
 * </p>
 * <p>
 * While this consistency of the IDs should be ensured by of the kernel (see {@code ModelContext}),
 * this constraint checker is an additional safety layer.
 * </p>
 * <p>
 * <b>CAVEAT:</b> For performance reasons, be sure to register this constraint checker only for a
 * class the exclusively represents the root of your model.
 * </p>
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 25605EB40BBE6C91C0E420FECB26F318
 */
public class IdConsistencyChecker extends ConstraintCheckerBase<EObject> {

	/** {@inheritDoc} */
	@Override
	public List<IdConsistencyConstraintViolation<IIdLabeled>> apply(EObject rootObject) {
		ArrayList<IdConsistencyConstraintViolation<IIdLabeled>> result =
				new ArrayList<IdConsistencyConstraintViolation<IIdLabeled>>();

		IPersistencyService ps = IPersistencyService.getInstance();
		ITopLevelElement topLevelElement = ps.getTopLevelElementFor(rootObject);
		if(topLevelElement != null) {
			for(IIdLabeled current : getChildrenWithType(rootObject, IIdLabeled.class)) {
				String persistedId = topLevelElement.getId(current);
				if(!(new Integer(current.getId()).toString()).equals(persistedId)) {
					result.add(createConsistencyIdViolation(current, persistedId));
				}
			}
		}
		return result;
	}
}
