/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.base;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.PrototypeCategory;

/**
 * Base implementation for {@link IPrototypeProvider}s.
 * This implementation is intended for <i>static</i> prototype lists, i.e., lists which are never
 * changing in the course of the execution. If you want dynamic lists of prototypes, implement
 * directly {@link IPrototypeProvider}.
 * <p>
 * Sub-classes must implement {@link #registerPrototypes()} to register specific prototypes using
 * {@link #registerPrototype(String, EObject, String)} and
 * {@link #registerPrimaryPrototype(String, EObject, String)}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 8152034EE41E27B3B9D250D89D9CADA2
 */
public abstract class PrototypeProviderBase implements IPrototypeProvider {

	/** Stores the registered prototypes. */
	protected final List<Prototype> prototypes = new ArrayList<Prototype>();

	/** Stores the prototypes categories. */
	protected final Map<String, PrototypeCategory> categories =
			new HashMap<String, PrototypeCategory>();

	/** Constructor. */
	public PrototypeProviderBase() {
		registerPrototypes();
	}

	/** Registers all prototypes. Sub-classes must override. */
	protected abstract void registerPrototypes();

	/** Registers the given {@link EObject} with the given name and category. */
	protected final void registerPrototype(String name, EObject prototype, String categoryName) {
		Prototype prototypeObject = new Prototype(name, prototype, false);
		prototypes.add(prototypeObject);
		registerElementInCategory(prototypeObject, categoryName);
	}

	/**
	 * Registers the given {@link EObject} with the given name as primary
	 * prototype with the given category.
	 */
	protected final void registerPrimaryPrototype(String name, EObject prototype,
			String categoryName) {
		Prototype prototypeObject = new Prototype(name, prototype, true);
		prototypes.add(prototypeObject);
		registerElementInCategory(prototypeObject, categoryName);
	}

	/** {@inheritDoc} */
	@Override
	public final List<Prototype> getPrototypes() {
		return CollectionUtils.asUnmodifiable(prototypes);
	}

	/** Registers the element with the given category. */
	private void registerElementInCategory(Prototype element, String category) {
		PrototypeCategory cat = registerPrototypeCategory(category);
		cat.add(element);
	}

	/** Registers an existing prototype category. */
	protected PrototypeCategory registerPrototypeCategory(String category) {
		PrototypeCategory cat = categories.get(category);
		if(cat == null) {
			cat = new PrototypeCategory(category);
			categories.put(category, cat);
		}
		return cat;
	}

	/** Registers an existing prototype category as child of a parent category. */
	protected PrototypeCategory
			registerPrototypeCategory(PrototypeCategory parent, String category) {
		PrototypeCategory cat = registerPrototypeCategory(category);
		parent.add(cat);
		cat.setPrototypeCategoryParent(parent);
		return cat;
	}

	/** {@inheritDoc} */
	@Override
	public List<PrototypeCategory> getCategories() {
		List<PrototypeCategory> result = new ArrayList<PrototypeCategory>();
		result.addAll(categories.values());
		return result;
	}
}
