/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.storage.eclipse;

import static java.util.Collections.emptyList;
import static java.util.Collections.unmodifiableList;
import static java.util.Collections.unmodifiableSet;
import static org.eclipse.core.resources.IResourceDelta.ADDED;
import static org.eclipse.core.resources.IResourceDelta.CHANGED;
import static org.eclipse.core.resources.IResourceDelta.REMOVED;
import static org.eclipse.core.resources.ResourcesPlugin.getWorkspace;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getBundle;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getConfigurationElements;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.loadClass;
import static org.fortiss.tooling.kernel.utils.LoggingUtils.error;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.IEclipseResourceStorageLocationProvider;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.extension.data.ModelStorageError;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectiveKernelService;
import org.fortiss.tooling.kernel.introspection.items.EclipseResourceStorageServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.service.IEclipseResourceStorageService;
import org.fortiss.tooling.kernel.service.IKernelIntrospectionSystemService;
import org.fortiss.tooling.kernel.service.IMigrationService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.osgi.framework.Bundle;

/**
 * This class implements the persistency service behavior for Eclipse file
 * storage.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 5434E8D36C6A296B0248B11235393FA3
 */
public class EclipseResourceStorageService implements IEclipseResourceStorageService,
		IResourceChangeListener, IResourceDeltaVisitor, IStorageProvider,
		IIntrospectiveKernelService {
	/** The singleton instance. */
	private static final EclipseResourceStorageService INSTANCE =
			new EclipseResourceStorageService();

	/** Returns singleton instance of the service. */
	public static EclipseResourceStorageService getInstance() {
		return INSTANCE;
	}

	/** The prototype provider extension point ID for location providers. */
	private static final String LOCATION_PROVIDER_EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.eclipseResourceStorageLocationProvider";

	/** The prototype provider configuration element name for location providers. */
	private static final String LOCATION_PROVIDER_CONFIGURATION_ELEMENT_NAME =
			"storageLocationProvider";

	/** Stores the registered storage providers. */
	private final List<IEclipseResourceStorageLocationProvider> storageProviderList =
			new ArrayList<IEclipseResourceStorageLocationProvider>();

	/** Cache of models loaded so far. */
	private final Map<IFile, ModelContext> loadedContexts = new HashMap<IFile, ModelContext>();

	/** List of failed model storage files. */
	private final List<ModelStorageError> errorFiles = new ArrayList<ModelStorageError>();

	/** Map from toplevel elements to context. */
	private final Map<EObject, ModelContext> rootElementContexts =
			new IdentityHashMap<EObject, ModelContext>();

	/** Starts the storage provider. */
	public void initializeService() {
		setupLocationProviders();
		searchWorkspaceForModels();
	}

	/** Starts the storage provider. */
	public void startService() {
		IPersistencyService.getInstance().registerModelStorageProvider(this);
		// register resource change listener last, since on-disk changes
		// are discovered during workspace search, thus introducing a possible
		// cyclic call during provider initialization
		getWorkspace().addResourceChangeListener(this);

		IKernelIntrospectionSystemService.getInstance().registerService(this);
	}

	/** Searches existing models in Eclipse project roots. */
	private synchronized void searchWorkspaceForModels() {
		for(IProject project : getWorkspace().getRoot().getProjects()) {
			if(!project.isOpen()) {
				continue;
			}
			searchResourceForModels(project);
		}
	}

	/** Searches the given container for models. */
	private void searchResourceForModels(IContainer container) {
		try {
			for(IResource res : container.members()) {
				if(res instanceof IFile) {
					IFile file = (IFile)res;
					if(checkLocationProvider(file)) {
						loadContext(file);
					}
				} else if(res instanceof IFolder) {
					searchResourceForModels((IFolder)res);
				}
			}
		} catch(CoreException e) {
			error(ToolingKernelActivator.getDefault(),
					"Core Exception while scanning Eclipse workspace", e);
		}
	}

	/**
	 * Searches all location providers and loads the model context if some
	 * location provider validates the given file as model storage file.
	 */
	private boolean checkLocationProvider(IFile file) {
		if(file.getFileExtension() == null) {
			return false;
		}
		for(IEclipseResourceStorageLocationProvider provider : storageProviderList) {
			if(provider.isStorageLocation(file)) {
				return true;
			}
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void resourceChanged(IResourceChangeEvent event) {
		synchronized(this) {
			if(event.getDelta() == null) {
				return;
			}
			try {
				event.getDelta().accept(this);
			} catch(final CoreException e) {
				error(ToolingKernelActivator.getDefault(), "Kernel Core Ecxeption!", e);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean visit(IResourceDelta delta) {
		if(delta.getResource() instanceof IFile) {
			runWorkspaceChangeJob((IFile)delta.getResource(), delta.getKind());
		}
		return true;
	}

	/** Runs resource change job as workspace job. */
	private void runWorkspaceChangeJob(final IFile file, final int changeKind) {
		new WorkspaceJob("ModelFileChangedJob") {
			@Override
			public IStatus runInWorkspace(IProgressMonitor monitor) {
				IPersistencyService ps = IPersistencyService.getInstance();
				synchronized(EclipseResourceStorageService.this) {
					// Model file that is managed by kernel has been changed
					if(checkLocationProvider(file)) {
						if(changeKind == ADDED) {
							loadContext(file);
						} else if(changeKind == REMOVED && isLoaded(file)) {
							unloadContext(file);
						} else if(changeKind == CHANGED) {
							if(isLoaded(file)) {
								handleChange(file);
							} else {
								loadContext(file);
							}
						}
						// Refresh the top-level elements
						ps.refreshTopLevelElements(EclipseResourceStorageService.this);
					} else if(changeKind == REMOVED) {
						// A file has been removed. Remove it from one of the ResourceSets owned by
						// the kernel.
						for(ITopLevelElement topLevelElement : ps.getTopLevelElements()) {
							final ResourceSet resourceSet = topLevelElement.getResourceSet();
							final List<Resource> toRemove = new ArrayList<Resource>();
							for(Resource resource : resourceSet.getResources()) {
								URI fileURI =
										URI.createPlatformResourceURI(
												file.getFullPath().toString(), true);
								if(resource.getURI().equals(fileURI)) {
									toRemove.add(resource);
								}
							}

							// Remove resource(s) corresponding to removed file
							topLevelElement.runAsCommand(new Runnable() {
								@Override
								public void run() {
									for(Resource resource : toRemove) {
										List<EObject> contents = new ArrayList<EObject>();
										contents.addAll(resource.getContents());
										for(EObject obj : contents) {
											EcoreUtil.delete(obj);
										}
										resource.unload();
										resourceSet.getResources().remove(resource);
									}
								}
							});
						}
					}
				}
				return Status.OK_STATUS;
			}
		}.schedule();
	}

	/** Handles a change of the given file (which is actually managed). */
	private void handleChange(IFile file) {
		ModelContext context = loadedContexts.get(file);
		// this avoids to reload the context when itself caused the disk change
		if(!context.getAndResetLastChangeWasCausedByModelContext()) {
			unloadContext(file);
			loadContext(file);
		}
	}

	/** Returns whether the given file has been loaded by this manager. */
	public synchronized boolean isLoaded(IFile file) {
		return loadedContexts.containsKey(file);
	}

	/** Loads a file and creates a new context from its contents. */
	private void loadContext(IFile file) {
		try {
			ModelContext mc = new ModelContext(file);
			loadedContexts.put(file, mc);
			rootElementContexts.put(mc.getRootModelElement(), mc);
			postLoadContext(mc);
		} catch(Exception ioex) {
			String msg = "Exception while loading model file: " + file.getName();
			errorFiles.add(new FileModelStorageError(msg, ioex, file));
			error(ToolingKernelActivator.getDefault(), msg, ioex);
		}
	}

	/** Post-actions after loading a model context */
	public void postLoadContext(ModelContext mc) {
		// Note: migration could be treated as a "post load" provider, but it is important that it
		// runs before the other providers so it is handled separately.
		IMigrationService migs = IMigrationService.getInstance();
		if(migs.needMigration(mc, mc.getUnknownFeatures())) {
			migs.migrate(mc, mc.getUnknownFeatures());
		}
	}

	/** Unloads the context and destroys it. */
	private void unloadContext(IFile file) {
		ModelContext context = loadedContexts.remove(file);
		if(context != null) {
			rootElementContexts.remove(context.getRootModelElement());
			context.destroy();
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<ITopLevelElement> getTopLevelElements() {
		List<ITopLevelElement> result = new ArrayList<ITopLevelElement>();
		result.addAll(rootElementContexts.values());
		return result;
	}

	/** Initializes the storage provider list from location provider plugin extensions. */
	private void setupLocationProviders() {
		for(IConfigurationElement ce : getConfigurationElements(
				LOCATION_PROVIDER_EXTENSION_POINT_NAME,
				LOCATION_PROVIDER_CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = getBundle(ce);
			try {
				Class<?> handlerClass = loadClass(ce.getAttribute("provider"), bundle);
				IEclipseResourceStorageLocationProvider provider =
						(IEclipseResourceStorageLocationProvider)handlerClass.getConstructor()
								.newInstance();
				storageProviderList.add(provider);
			} catch(Exception ex) {
				error(ToolingKernelActivator.getDefault(), ex.getMessage(), ex);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IFile> getModelElementResources() {
		return new ArrayList<IFile>(loadedContexts.keySet());
	}

	/** {@inheritDoc} */
	@Override
	public List<ModelStorageError> getStorageErrors() {
		checkFiles();
		return errorFiles;
	}

	/** Checks the erroneous files for existence. */
	private void checkFiles() {
		List<ModelStorageError> iter = new ArrayList<ModelStorageError>(errorFiles);
		for(ModelStorageError err : iter) {
			IFile file = ((FileModelStorageError)err).file;
			if(loadedContexts.containsKey(file) || !file.exists()) {
				errorFiles.remove(err);
			}
		}
	}

	/** Class for file-based model errors. */
	private static class FileModelStorageError extends ModelStorageError {
		/** The file. */
		public IFile file;

		/** Constructor. */
		public FileModelStorageError(String msg, Exception cause, IFile file) {
			super(msg, cause);
			this.file = file;
		}
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Eclipse Resource Storage Provider";
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IIntrospectionItem> getIntrospectionItems() {
		return emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		return new EclipseResourceStorageServiceIntrospectionDetailsItem(
				unmodifiableList(storageProviderList), unmodifiableSet(loadedContexts.keySet()),
				unmodifiableList(errorFiles));
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		return getIntrospectionLabel() +
				"\n\nThis service storage implementation registered with the persistency service." +
				"\nIt searches the Eclipse workspace directory and its sub-directories for model files and" +
				"\ncreates top-level elements for it." +
				"\nAll existing files are presented to registered instances of IEclipseResourceStorageLocationProvider," +
				"\nwhich may then decide if the file extension is correct." +
				"\n\nThe service extension point is '" + LOCATION_PROVIDER_EXTENSION_POINT_NAME +
				"'.";
	}
}
