/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service.base;

import static java.util.Collections.emptyList;
import static org.conqat.lib.commons.reflect.ReflectionUtils.performNearestClassLookup;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getBundle;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getConfigurationElements;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.loadClass;
import static org.fortiss.tooling.kernel.utils.LoggingUtils.error;
import static org.fortiss.tooling.kernel.utils.LoggingUtils.warning;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.internal.ConnectionCompositorService;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectiveKernelService;
import org.osgi.framework.Bundle;

/**
 * Service base implementation, which supports handler registration with a set of {@link Object}
 * classes.
 * 
 * <p>
 * Sub-classes must implement:
 * <UL>
 * <LI>{@link #getExtensionPointName()} to provide the extension point ID,
 * <LI>{@link #getConfigurationElementName()} to provide the configuration element ID,
 * <LI>{@link #getHandlerClassAttribute()} to provide the attribute of the handler class.
 * </UL>
 * 
 * <p>
 * Sub-classes may override:
 * <UL>
 * <LI>{@link #getClassConfigurationElement()} to provide a element class configuration element,
 * <LI>{@link #getClassAttribute()} to provide a element class attribute.
 * <LI> {@link #requiresUniqueHandler()} should return true if each Object class must not have more
 * than one handler. The default is <code>false</code>.
 * </UL>
 * 
 * An example implementation of a service based on this class is given by
 * {@link ConnectionCompositorService}. The corresponding extension point schema is defined in
 * <code>schema/modelElementConnectionCompositor.exsd</code>.
 * 
 * <p>
 * This class implements {@link IIntrospectiveKernelService}, thus requiring all such services to
 * take part in the kernel introspection service system. Sub-classes must implement
 * {@link #getIntrospectionLabel()} and {@link #getIntrospectionDescription()}.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: ADF46952A610682E89FD917D0AD21203
 */
public abstract class ObjectAwareServiceBase<T extends IObjectAware<? extends Object>> implements
		IIntrospectiveKernelService {

	/** Stores the class to handler map. */
	protected final Map<Class<?>, List<T>> handlerMap = new HashMap<Class<?>, List<T>>();

	/** Stores the set of handlers independent of the registration class. */
	protected final Set<T> allHandlersSet = new HashSet<T>();

	/**
	 * Returns the list of registered handlers for the given class.
	 * 
	 * IMPORTANT: this method only returns the first registered handler which is found. A different
	 * implementation where all registered handlers are returned can be found in
	 * org.fortiss.tooling.base.annotation.AnnotationValueService#getRegisteredHandlers.
	 * If this behavior will be reused we should change this method. (Refs 2270)
	 */
	public List<T> getRegisteredHandlers(Class<?> modelElementClass) {
		return performNearestClassLookup(modelElementClass, handlerMap);
	}

	/** Returns all handlers as a set. */
	public Set<T> getAllHandlers() {
		return allHandlersSet;
	}

	/**
	 * Initializes the service by setting up the handler map. Sub-classes may override, but must
	 * call super implementation.
	 */
	public void initializeService() {
		setupHandlerMap();
	}

	/** Sarts the service. Sub-classes may override. */
	public void startService() {
		// the default does nothing.
	}

	/** Initializes the handler map from plugin extensions. */
	@SuppressWarnings({"unchecked"})
	private void setupHandlerMap() {
		for(IConfigurationElement ce : getConfigurationElements(getExtensionPointName(),
				getConfigurationElementName())) {
			Bundle bundle = getBundle(ce);
			try {
				Class<?> handlerClass =
						loadClass(ce.getAttribute(getHandlerClassAttribute()), bundle);
				T handler = (T)handlerClass.getConstructor().newInstance();

				for(IConfigurationElement ce2 : ce.getChildren(getClassConfigurationElement())) {
					Class<?> modelElementClass =
							loadClass(ce2.getAttribute(getClassAttribute()), bundle);
					addHandler(modelElementClass, handler);
				}
			} catch(Exception ex) {
				error(ToolingKernelActivator.getDefault(), ex.getMessage(), ex);
			}
		}
	}

	/** Adds the given handler to the map and the set. */
	protected void addHandler(Class<?> modelElementClass, T handler) {
		List<T> list = handlerMap.get(modelElementClass);
		if(list == null) {
			list = new LinkedList<T>();
			handlerMap.put(modelElementClass, list);
		}
		if(requiresUniqueHandler() && !list.isEmpty()) {
			warning(ToolingKernelActivator.getDefault(),
					"Encountered more than one handler registered with " + getExtensionPointName() +
							"(" + handler.getClass() + "," + list.get(0).getClass() + ")");
		}
		list.add(handler);
		allHandlersSet.add(handler);
	}

	/** Returns the extension point name. */
	protected abstract String getExtensionPointName();

	/** Returns the configuration element name. */
	protected abstract String getConfigurationElementName();

	/** Returns the handler class attribute name. */
	protected abstract String getHandlerClassAttribute();

	/**
	 * Returns whether the service requires a unique handler per model element
	 * class. By default more than one handler can be registered per model
	 * element class.
	 */
	protected boolean requiresUniqueHandler() {
		return false;
	}

	/**
	 * Returns the configuration element name for the object class. Sub-classes
	 * may override, but should first consider to use the default value
	 * <i>objectClass</i> in the extension point definition.
	 */
	protected String getClassConfigurationElement() {
		return "objectClass";
	}

	/**
	 * Returns the attribute name for the object class. Sub-classes may
	 * override, but should first consider to use the default value
	 * <i>objectClass</i> in the extension point definition.
	 */
	protected String getClassAttribute() {
		return "objectClass";
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IIntrospectionItem> getIntrospectionItems() {
		return emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		return null;
	}
}
