/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.style;

import static java.util.Objects.requireNonNull;
import static org.fortiss.tooling.spiderchart.style.FontStyle.BLACK_VERDANA_12PT;
import static org.fortiss.tooling.spiderchart.style.FontStyle.BLACK_VERDANA_8PT;
import static org.fortiss.tooling.spiderchart.style.LineStyle.SOLID_BLACK_1PT;

import java.text.DecimalFormat;

import org.fortiss.tooling.spiderchart.model.DoubleAxis;

/**
 * Class for all style attributes for {@link DoubleAxis spider chart axes}.
 * <P>
 * If {@link #segmentStyle} is non-null then indicators will be drawn.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 26797EA6472BA45A84B0F63F3AE92338
 */
public final class AxisStyle {

	/** Default axis style with solid black line, a single segment, and Verdana fonts. */
	public static final AxisStyle DEFAULT = new AxisStyle(SOLID_BLACK_1PT, BLACK_VERDANA_12PT, 1,
			BLACK_VERDANA_8PT);

	/** The line style. */
	private LineStyle lineStyle;
	/** The axis label style. */
	private FontStyle labelStyle;
	/** The number of segments to be drawn. */
	private int segments;
	/** The font style of the segment labels. */
	private FontStyle segmentStyle;
	/** The decimal formatter of the segment labels of the axis. */
	private DecimalFormat decimalFormat;
	/** Flag for drawing axis segment indicators. */
	private boolean useSegmentIndicators = true;

	/** Constructor. */
	public AxisStyle(LineStyle lineStyle, FontStyle labelStyle) {
		this(lineStyle, labelStyle, 0, null, new DecimalFormat("#.##"), true);
	}

	/** Constructor. */
	public AxisStyle(LineStyle lineStyle, FontStyle labelStyle, int numSegments,
			FontStyle segmentStyle) {
		this(lineStyle, labelStyle, numSegments, segmentStyle, new DecimalFormat("#.##"), true);
	}

	/** Constructor. */
	public AxisStyle(LineStyle lineStyle, FontStyle labelStyle, int numSegments,
			FontStyle segmentStyle, DecimalFormat decimalFormat) {
		this(lineStyle, labelStyle, numSegments, segmentStyle, decimalFormat, true);
	}

	/** Constructor. */
	public AxisStyle(LineStyle lineStyle, FontStyle labelStyle, int numSegments,
			FontStyle segmentStyle, DecimalFormat decimalFormat, boolean useSegmentIndicators) {
		this.lineStyle = requireNonNull(lineStyle);
		this.labelStyle = requireNonNull(labelStyle);
		this.segments = Math.max(1, numSegments);
		this.segmentStyle = segmentStyle;
		this.decimalFormat = decimalFormat;
		this.useSegmentIndicators = useSegmentIndicators;
	}

	/** Returns the line style. */
	public LineStyle getLineStyle() {
		return lineStyle;
	}

	/** Returns the label style. */
	public FontStyle getLabelStyle() {
		return labelStyle;
	}

	/** Returns the number of segments. */
	public int getSegments() {
		return segments;
	}

	/** Returns the segment style. */
	public FontStyle getSegmentStyle() {
		return segmentStyle;
	}

	/** Returns the decimal format. */
	public DecimalFormat getDecimalFormat() {
		return decimalFormat;
	}

	/** Returns whether to use segment indicators. */
	public boolean isUseSegmentIndicators() {
		return useSegmentIndicators;
	}
}
