/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.selection;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import org.conqat.lib.commons.assertion.CCSMPre;
import org.conqat.lib.commons.assertion.PreconditionException;

/**
 * Utility methods for dealing with selections.
 * 
 * @author hummelb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: 423C3D4F7D7F00FF6583B4CF748CA886
 */
public class SelectionUtils {

	/**
	 * Returns the first element of a selection or <code>null</code> if the
	 * selection is empty. This raises a {@link PreconditionException} if the
	 * provided selection is not an instance of {@link IStructuredSelection} or
	 * the element is not of the required type.
	 */
	@SuppressWarnings("unchecked")
	public static <T> T checkAndPickFirst(ISelection selection, Class<T> type) {
		CCSMPre.isInstanceOf(selection, IStructuredSelection.class);
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		if (structuredSelection.isEmpty()) {
			return null;
		}

		Object firstElement = structuredSelection.getFirstElement();
		CCSMPre.isInstanceOf(firstElement, type);
		return (T) firstElement;
	}

	/**
	 * Returns the first element of a selection or <code>null</code> if the
	 * selection is empty. This return <code>null</code> if the provided
	 * selection is not an instance of {@link IStructuredSelection} or the
	 * element is not of the required type.
	 */
	@SuppressWarnings("unchecked")
	public static <T> T checkAndPickFirstSafe(ISelection selection,
			Class<T> type) {
		if (!(selection instanceof IStructuredSelection)) {
			return null;
		}
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		if (structuredSelection.isEmpty()) {
			return null;
		}

		Object firstElement = structuredSelection.getFirstElement();
		if (!type.isInstance(firstElement)) {
			return null;
		}
		return (T) firstElement;
	}

	/**
	 * Returns a list of all the selected elements that have the given type.
	 * This raises a {@link PreconditionException} if the provided selection is
	 * not an instance of {@link IStructuredSelection}.
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> selectAll(ISelection selection, Class<T> type) {
		CCSMPre.isInstanceOf(selection, IStructuredSelection.class);
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		List<T> elements = new ArrayList<T>();
		for (Object element : structuredSelection.toList()) {
			if (type.isInstance(element)) {
				elements.add((T) element);
			}
		}
		return elements;
	}

	/**
	 * Returns all elements of a selection or an empty list if the selection is
	 * empty. This raises a {@link PreconditionException} if the provided
	 * selection is not an instance of {@link IStructuredSelection} or the
	 * elements are not of the required type.
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> checkAndSelectAll(ISelection selection,
			Class<T> type) {
		CCSMPre.isInstanceOf(selection, IStructuredSelection.class);
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;

		List<T> elements = structuredSelection.toList();
		for (T element : elements) {
			CCSMPre.isInstanceOf(element, type);
		}
		return elements;
	}
}